/*

	xwailer.c

	The Wailer - version X Window

	(C) Angel Ortega 1995

*/


/* Cabeceras standard */

#include <stdlib.h>
#include <unistd.h>


/* Cabeceras de X Intrinsics y Widgets */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Shell.h>

/* nombre de la aplicacion (necesario para waimotor.c) */

char * szApp="The X Wailer 1.2 beta";


/* motor gestor de mensajes */

#include "waimotor.c"


/* Widgets */

/* contexto de aplicacion */

XtAppContext ap;

/* Widget shell y box principal */

Widget wshell, wmain;

/* Widget de la lista y del cuerpo del mensaje */

Widget wlist, wtexto;

/* Widget box de las opciones y opciones */
Widget wopciones;
Widget wdelmsg,wnewmsg,wreplymsg;
Widget wnextmsg,wnextconf,wprevmsg,wprevconf;
Widget wacerca, wnuehis, wgo;

/* Widget de mensaje */

Widget wmsg;


/* Widget de las ventanas de edicion */

Widget edit_shell, edit_box, edit_conf, edit_text,
	edit_subj, edit_to, edit_ok, edit_cancel;


/* tamanos */

#define LST_WIDTH 540
#define LST_HEIGHT 70
#define TXT_WIDTH LST_WIDTH+10
#define TXT_HEIGHT 250


/* buffer general (para mensajes y eso) */

char waibuffer[16384];


/* cadenas de transformacion Ascii <-> IBM PC <-> ISO_8859_1 */

char * SpainAscii="aeiouunN?!";
char * SpainIBM="\xa0\x82\xa1\xa2\xa3\x81\xa4\xa5\xa8\xad";
char * SpainISO="\xe1\xe9\xed\xf3\xfa\xfc\xf1\xd1\xbf\xa1";


/* la lista visible */

#define NUM_LST_VIS_STRINGS 4
char * LstStrings[NUM_LST_VIS_STRINGS];


/* la lista real: lista encadenada de elementos */

#define LST_ELEM_STR 80
struct lstelem
{
	char str[LST_ELEM_STR];
	struct lstelem * next;
};

struct lstelem * LstFirst=NULL;

/* argc y argv de main */
int Argc;
char * Argv[30];


/* fuente fino */
XFontStruct * font_thin;


/***********************
	Funciones
************************/


char TransChar(char c, char * org, char * des)
/* transforma c segun org y des */
{
	while(*org)
	{
		if(c==*org)
		{
			c=*des;
			break;
		}

		org++;
		des++;
	}

	return(c);
}


void TransStr(char * str, char * org, char * des)
/* traduce la cadena entera segun org y des */
{
	while(*str)
	{
		*str=TransChar(*str,org,des);
		str++;
	}
}


void procstr_read(char * cadena)
/* funcion procstr para ReadMsg */
{
	TransStr(cadena, SpainIBM, SpainISO);
	strcat(cadena,"\n");
}


void RepintarLista(void)
/* redibuja los elementos de la lista */
{
	struct lstelem * elem;
	int n,m;

	m=Mensaje-NUM_LST_VIS_STRINGS+1;
	if(m<0) m=0;

	/* se salta tantos como indique el numero de mensaje */
	for(n=0,elem=LstFirst;n<m;n++)
	{
		if(elem->next!=NULL)
			elem=elem->next;
	}

	/* se rellenan tantos strings de la lista como quepan */
	for(n=0;n<NUM_LST_VIS_STRINGS;n++)
	{
		if(elem==NULL)
			LstStrings[n]="";
		else
		{
			LstStrings[n]=elem->str;
			elem=elem->next;
		}
	}

	/* cambia la lista */
	XawListChange(wlist,LstStrings,NUM_LST_VIS_STRINGS,
		LST_WIDTH,0);

	/* selecciona el elemento */
	XawListHighlight(wlist,Mensaje<NUM_LST_VIS_STRINGS?Mensaje:
		NUM_LST_VIS_STRINGS-1);
}


void CargarMensaje(void)
/* carga el texto en el mensaje actual */
{
	int x=-1;
	char tmp[2048];

	if(Conferencia!=-1 && Mensaje!=NO_MENSAJE)
		x=ReadMsg(NULL,0,waibuffer,sizeof(waibuffer)-1,
			procstr_read,NULL);
	else
		waibuffer[0]=0;

	XtVaSetValues(wtexto,"string",waibuffer,NULL);

	sprintf(tmp,"[%s] Conferencia: %s Estado: %s",
		Histo==1?"Histrico":"Nuevos",
		Conferencia==-1?"Ninguna":Conferencias[Conferencia],
		NumMsgs?(Msg.dat&MASK_BOR)?"borrado":
			(Msg.dat&MASK_LEI?"ya ledo":"nuevo"):"-");

	XtVaSetValues(wmsg,"label",tmp,"width",TXT_WIDTH,NULL);

	if(Mensaje!=NO_MENSAJE)
		SetMsgFlags(MASK_LEI);
}


void NewElem(void)
/* funcion NewElem para ReadList */
{
	struct lstelem * elem;
	struct lstelem * lastelem;
	char tmp[1024];

	sprintf(tmp,"%10s -> %10s %s",Remit,To,Subject);
	TransStr(tmp,SpainIBM,SpainISO);

	elem=(struct lstelem *) malloc(sizeof(struct lstelem));

	if(elem!=NULL)
	{
		strncpy(elem->str,tmp,LST_ELEM_STR);
		elem->next=NULL;

		if(LstFirst==NULL)
			LstFirst=elem;
		else
		{
			for(lastelem=LstFirst;lastelem->next!=NULL;
				lastelem=lastelem->next);

			lastelem->next=elem;
		}
	}
}


CargarLista(void)
/* carga la lista */
{
	struct lstelem * elem;
	struct lstelem * nextelem;

	/* libera todos los elementos */
	for(elem=LstFirst;elem;elem=nextelem)
	{
		nextelem=elem->next;
		free(elem);
	}
	LstFirst=NULL;

	ReadList(NewElem);

	if(NumMsgs)
		Mensaje=0;
	else
		Mensaje=NO_MENSAJE;

	RepintarLista();
}


void PrevConf(Widget W, XtPointer client_data, XtPointer call_data)
/* anterior conferencia */
{
	if(Conferencia>0)
	{
		do
		{
			Conferencia--;

			CargarLista();
		} while (NumMsgs==0 && Conferencia>0);

		CargarMensaje();
	}
}


void PrevMsg(Widget W, XtPointer client_data, XtPointer call_data)
/* anterior mensaje */
{
	if(Mensaje>0)
	{
		Mensaje--;

		CargarMensaje();

		RepintarLista();
	}
	else
		PrevConf(W, client_data, call_data);
}


void NextConf(Widget W, XtPointer client_data, XtPointer call_data)
/* siguiente conferencia */
{
	if(Conferencia<NumConfes-1)
	{
		do
		{
			Conferencia++;

			CargarLista();

		} while (NumMsgs==0 && Conferencia<NumConfes-1);

		CargarMensaje();
	}
}


void NextMsg(Widget W, XtPointer client_data, XtPointer call_data)
/* siguiente mensaje */
{
	if(Mensaje<NumMsgs-1)
	{
		Mensaje++;

		CargarMensaje();

		RepintarLista();
	}
	else
		NextConf(W, client_data, call_data);
}


void DelMsg(Widget W, XtPointer client_data, XtPointer call_data)
/* borra el mensaje actual */
{
	if(NumMsgs)
	{
		SetMsgFlags(MASK_BOR);

		NextMsg(W,client_data,call_data);
	}
}


void EndWailer(Widget w, XtPointer client_data, XtPointer call_data)
/* sale del X Wailer */
{
	EndW();

	exit(0);
}


void EditCancel(Widget w, XtPointer client_data, XtPointer call_data)
/* cancela una ventana de edicion */
{
	exit(0);
}


void EditOK(Widget w, XtPointer client_data, XtPointer call_data)
/* confirma un mensaje editado */
{
	char * ptr;
	char ConfStr[MAX_NOM_CONF+1];

	/* coge los nuevos valores */
	XtVaGetValues(edit_conf,"string",&ptr,NULL);
	strcpy(ConfStr,ptr);

	XtVaGetValues(edit_text,"string",&ptr,NULL);
	strcpy(waibuffer,ptr);

	XtVaGetValues(edit_subj,"string",&ptr,NULL);
	strcpy(Subject,ptr);

	XtVaGetValues(edit_to,"string",&ptr,NULL);
	strcpy(To,ptr);

	TransStr(waibuffer,SpainISO,SpainIBM);

	WriteMsg(waibuffer,ConfStr,NULL,NULL);

	exit(0);
}


void CuadroEdicion(void)
/* abre un cuadro de edicion de mensaje */
/* Esta funcion hace un fork() y se convierte en un nuevo proceso */
{
	static NumResp=0;
	XtAppContext edit_app;
	char tmp[50];

	NumResp++;

	if(!fork())
	{
		sprintf(tmp,"The X Wailer - Respuesta %d", NumResp);
		Argv[0]=tmp;

		/* crea un nuevo widget shell */
		edit_shell=XtVaAppInitialize(&edit_app,"Xwailer",NULL,0,
			&Argc,Argv,NULL,NULL);

		edit_box=XtVaCreateManagedWidget("xwailerResp",
			boxWidgetClass,edit_shell,NULL);

		XtVaCreateManagedWidget("Conferencia",labelWidgetClass,
			edit_box,NULL);
		edit_conf=XtVaCreateManagedWidget("Conferencia       ",
			asciiTextWidgetClass,edit_box,
			"string",Conferencias[Conferencia],
			"editType",XawtextEdit,"font",font_thin,
			NULL);

		XtVaCreateManagedWidget("A",labelWidgetClass,
			edit_box,NULL);
		edit_to=XtVaCreateManagedWidget("AStr",
			asciiTextWidgetClass,edit_box,
			"string",To,
			"editType",XawtextEdit,"font",font_thin,
			NULL);

		XtVaCreateManagedWidget("Tema",labelWidgetClass,
			edit_box,NULL);
		edit_subj=XtVaCreateManagedWidget("TemaStr",
			asciiTextWidgetClass,edit_box,
			"string",Subject,
			"editType",XawtextEdit,
			"width",TXT_WIDTH,"font",font_thin,
			NULL);

		edit_text=XtVaCreateManagedWidget("EditText",
			asciiTextWidgetClass,edit_box,
			"width",TXT_WIDTH,"height",TXT_HEIGHT,"scrollVertical",
			XawtextScrollWhenNeeded,"wrap",True,
			"editType",XawtextEdit,"string",waibuffer,
			"font",font_thin,NULL);
			
		edit_ok=XtVaCreateManagedWidget(" OK ",
			commandWidgetClass,edit_box,NULL);

		edit_cancel=XtVaCreateManagedWidget("Cancelar",
			commandWidgetClass,edit_box,NULL);

		XtAddCallback(edit_ok, "callback", EditOK, 0);
		XtAddCallback(edit_cancel, "callback", EditCancel, 0);

		XtRealizeWidget(edit_shell);

		XtAppMainLoop(edit_app);
	}
}


void NewMsg(Widget w, XtPointer client_data, XtPointer call_data)
/* edita un mensaje nuevo */
{
	Subject[0]=0;
	To[0]=0;
	waibuffer[0]=0;

	CuadroEdicion();
}


void ReplyMsg(Widget w, XtPointer client_data, XtPointer call_data)
/* edita un mensaje con quotes */
{
	if(NumMsgs)
	{
		QuoteMsg(waibuffer,sizeof(waibuffer)-1,procstr_read);

		strcpy(To, Remit);

		CuadroEdicion();
	}
}


void NueHis(Widget w, XtPointer client_data, XtPointer call_data)
/* cambia entre los nuevos y el histrico */
{
	Histo=!Histo;

	XtVaSetValues(wnuehis,"label",
		Histo==0?" Histrico  ":"   Nuevos   ",NULL);

	CargarLista();
	CargarMensaje();
}


void AcercaDe(Widget w, XtPointer client_data, XtPointer call_data)
/* cuadro acerca de */
{
	XtAppContext acerca_app;
	Widget acerca_shell, acerca_box, acerca_ok;

	if(fork()==0)
	{
		Argv[0]="Acerca de X-Wailer";

		acerca_shell=XtVaAppInitialize(&acerca_app,"Xwailer",NULL,0,
			&Argc,Argv,NULL,NULL);

		acerca_box=XtVaCreateManagedWidget("xwailerAbout",
			boxWidgetClass,acerca_shell,NULL);

		XtVaCreateManagedWidget("AcercaOk",
			labelWidgetClass,acerca_box,
			"label",
			"The Wailer for X Window\n"\
			"Lector de correo offline para CRCOK BBS\n\n"\
			"(Linux/XFree86/Athena Widgets)\n\n"\
			"(C) Angel Ortega 1995 johannes@bbs.crcok.uucp",
			NULL);

		acerca_ok=XtVaCreateManagedWidget(" OK ",
			commandWidgetClass,acerca_box,NULL);

		XtAddCallback(acerca_ok, "callback", EditCancel, 0);

		XtRealizeWidget(acerca_shell);

		XtAppMainLoop(acerca_app);
	}
}


void HitList(Widget w, XtPointer client_data, XtPointer call_data)
/* procesa los clicks sobre la lista */
{
	RepintarLista();
}


void WaiMask(void)
/* crea la mascara */
{
	Display * disp;
	Widget w2;

	/* Widget global */
	wmain=XtVaCreateManagedWidget("xwailer",
		boxWidgetClass,wshell,NULL);

	/* coge el display */
	disp=XtDisplay(wmain);

	/* carga en l el fuente de edicin */
	font_thin=XLoadQueryFont(disp, "variable");

	/* Lista */
	wlist=XtVaCreateManagedWidget("Lista",listWidgetClass,wmain,
		"defaultColumns",1,"longest",LST_WIDTH,
		"height",LST_HEIGHT,NULL);

	/* Texto */
	wtexto=XtVaCreateManagedWidget("Texto",asciiTextWidgetClass,wmain,
		"width",TXT_WIDTH,"height",TXT_HEIGHT,"scrollVertical",
		XawtextScrollWhenNeeded,"wrap",True,"font",font_thin,NULL);

	/* opciones */
	w2=XtVaCreatePopupShell("ToolBox",
		transientShellWidgetClass,wmain,
		NULL);

	wopciones=XtVaCreateManagedWidget("Opciones",
		boxWidgetClass,w2,"orientation",XtorientVertical,
		NULL);

	wdelmsg=XtVaCreateManagedWidget("   Borrar   ",commandWidgetClass,
		wopciones,NULL);

	wreplymsg=XtVaCreateManagedWidget(" Contestar  ",
		commandWidgetClass,wopciones,NULL);

	wnewmsg=XtVaCreateManagedWidget("  Escribir  ",
		commandWidgetClass,wopciones,NULL);

	wprevconf=XtVaCreateManagedWidget("<<",commandWidgetClass,
		wopciones,NULL);
	wprevmsg=XtVaCreateManagedWidget("<",commandWidgetClass,wopciones,
		NULL);
	wnextmsg=XtVaCreateManagedWidget(">",commandWidgetClass,wopciones,
		NULL);
	wnextconf=XtVaCreateManagedWidget(">>",commandWidgetClass,wopciones,
		NULL);

	wnuehis=XtVaCreateManagedWidget(" Histrico  ",commandWidgetClass,
		wopciones,NULL);
	wacerca=XtVaCreateManagedWidget("Acerca de...",commandWidgetClass,
		wopciones,NULL);
	wgo=XtVaCreateManagedWidget("   Salir    ",commandWidgetClass,wopciones,
		NULL);

	/* callbacks */

	XtAddCallback(wlist, "callback", HitList, 0);
	XtAddCallback(wdelmsg, "callback", DelMsg, 0);
	XtAddCallback(wnewmsg, "callback", NewMsg, 0);
	XtAddCallback(wreplymsg, "callback", ReplyMsg, 0);
	XtAddCallback(wnextmsg, "callback", NextMsg, 0);
	XtAddCallback(wprevmsg, "callback", PrevMsg, 0);
	XtAddCallback(wnextconf, "callback", NextConf, 0);
	XtAddCallback(wprevconf, "callback", PrevConf, 0);
	XtAddCallback(wgo, "callback", EndWailer, 0);
	XtAddCallback(wacerca, "callback", AcercaDe, 0);
	XtAddCallback(wnuehis, "callback", NueHis, 0);

	/* widget de mensaje */
	wmsg=XtVaCreateManagedWidget("Mensaje",labelWidgetClass,wmain,
		"label","CRCOK BBS 34-1-3144075",
		"width",TXT_WIDTH,"font",font_thin,NULL);

	XtRealizeWidget(wshell);

	XtPopup(w2, XtGrabNone);
}


int main(int argc, char * argv[])
{
	int n;

	/* modifica el nombre de la ventana */
	argv[0]=szApp;

	/* guarda los parametros de main */
	for(Argc=argc,n=0;n<Argc;n++)
		Argv[n]=argv[n];

	/* coge el Yo: como el nombre de login */
	strcpy(Yo,getlogin());

	/* cambia al HOME del usuario */
	chdir(getenv("HOME"));

	/* cambia al directorio .wailer */
	if(chdir(".wailer")==-1)
	{
		if(mkdir(".wailer",0777)==-1)
		{
			printf("Error creando directorio .wailer\n");
			return(0);
		}

		chdir(".wailer");
	}

	/* inicializa el X Toolkit */
	wshell=XtVaAppInitialize(&ap,"Xwailer",NULL,0,
		&argc,argv,NULL,NULL);

	WaiMask();

	if(InitW())
	{
		printf("Procesando nuevos...\n");
		ProcessNew();

		if(IsRespFile())
		{
			printf("Inicializando respuestas...\n");
			InitRespFile();
		}
	}

	Conferencia=Mensaje=0;
	CargarLista();
	CargarMensaje();

	XtAppMainLoop(ap);

	return(0);
}

